#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>
#include <QtCore/qhash.h>
#include <QtCore/qstring.h>

namespace QmlCacheGeneratedCode {
namespace _qt_qml_org_kde_klevernotes_contents_ui_Main_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_DeleteConfirmationDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_FilePickerDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_FileSaverDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_FolderPickerDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_FontPickerDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_ItemPropertiesDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_LeavePaintingDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_LinkDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_LinkNoteDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_MoveDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_NamingDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_NamingErrorDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_PrintingDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_StorageDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_StorageConversionDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_TextDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_URLDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiGrid_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiPicker_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiTonesPicker_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_imagePickerDialog_ButtonDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_imagePickerDialog_ImagePickerDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_tableMakerDialog_TableMakerDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_tableMakerDialog_UpDownButtonDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_todoDialog_TextAreaDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_dialogs_todoDialog_ToDoDialog_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_pages_AboutPage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_pages_MainPage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_pages_PaintingPage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_pages_PrintingPage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_pages_SettingsPage_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_painting_ColorBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_painting_ColorButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_painting_DrawingToolBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_painting_DrawingToolBarOptions_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_components_ClickableArrow_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_components_ClickableIcon_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_components_ActionListViewDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_DisplayPreview_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_EditorHighlightPreview_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_EditorTab_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_FontPicker_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_GeneralTab_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_NotePreviewTab_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_PluginsTab_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_SettingsColorButton_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_settings_TabBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_ActionsList_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_ExpandingFormSwitch_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_KAction_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_ReversibleButtonDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sideBar_ActionBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sideBar_ContextMenu_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sideBar_SearchBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sideBar_Sidebar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sideBar_TreeItem_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_sideBar_TreeView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_components_CheatSheetEntry_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_components_NotesMapEntry_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_BottomToolBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_CheatSheet_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_EditorView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_NotesMap_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_TextDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_TextEditor_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_textEditor_TextToolBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_todoEditor_TodoDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_org_kde_klevernotes_contents_ui_todoEditor_ToDoView_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/Main.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_Main_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/DeleteConfirmationDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_DeleteConfirmationDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/FilePickerDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_FilePickerDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/FileSaverDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_FileSaverDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/FolderPickerDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_FolderPickerDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/FontPickerDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_FontPickerDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/ItemPropertiesDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_ItemPropertiesDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/LeavePaintingDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_LeavePaintingDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/LinkDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_LinkDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/LinkNoteDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_LinkNoteDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/MoveDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_MoveDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/NamingDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_NamingDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/NamingErrorDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_NamingErrorDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/PrintingDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_PrintingDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/StorageDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_StorageDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/StorageConversionDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_StorageConversionDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/TextDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_TextDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/URLDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_URLDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/emojiDialog/EmojiDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/emojiDialog/EmojiDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/emojiDialog/EmojiGrid.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiGrid_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/emojiDialog/EmojiPicker.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiPicker_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/emojiDialog/EmojiTonesPicker.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_emojiDialog_EmojiTonesPicker_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/imagePickerDialog/ButtonDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_imagePickerDialog_ButtonDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/imagePickerDialog/ImagePickerDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_imagePickerDialog_ImagePickerDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/tableMakerDialog/TableMakerDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_tableMakerDialog_TableMakerDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/tableMakerDialog/UpDownButtonDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_tableMakerDialog_UpDownButtonDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/todoDialog/TextAreaDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_todoDialog_TextAreaDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/dialogs/todoDialog/ToDoDialog.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_dialogs_todoDialog_ToDoDialog_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/pages/AboutPage.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_pages_AboutPage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/pages/MainPage.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_pages_MainPage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/pages/PaintingPage.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_pages_PaintingPage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/pages/PrintingPage.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_pages_PrintingPage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/pages/SettingsPage.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_pages_SettingsPage_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/painting/ColorBar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_painting_ColorBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/painting/ColorButton.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_painting_ColorButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/painting/DrawingToolBar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_painting_DrawingToolBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/painting/DrawingToolBarOptions.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_painting_DrawingToolBarOptions_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/components/ClickableArrow.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_components_ClickableArrow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/components/ClickableIcon.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_components_ClickableIcon_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/components/ActionListViewDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_components_ActionListViewDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/DisplayPreview.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_DisplayPreview_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/EditorHighlightPreview.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_EditorHighlightPreview_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/EditorTab.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_EditorTab_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/FontPicker.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_FontPicker_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/GeneralTab.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_GeneralTab_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/NotePreviewTab.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_NotePreviewTab_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/PluginsTab.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_PluginsTab_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/SettingsColorButton.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_SettingsColorButton_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/settings/TabBar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_settings_TabBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sharedComponents/ActionsList.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_ActionsList_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sharedComponents/ExpandingFormSwitch.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_ExpandingFormSwitch_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sharedComponents/KAction.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_KAction_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sharedComponents/ReversibleButtonDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sharedComponents_ReversibleButtonDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sideBar/ActionBar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sideBar_ActionBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sideBar/ContextMenu.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sideBar_ContextMenu_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sideBar/SearchBar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sideBar_SearchBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sideBar/Sidebar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sideBar_Sidebar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sideBar/TreeItem.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sideBar_TreeItem_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/sideBar/TreeView.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_sideBar_TreeView_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/components/CheatSheetEntry.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_components_CheatSheetEntry_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/components/NotesMapEntry.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_components_NotesMapEntry_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/BottomToolBar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_BottomToolBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/CheatSheet.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_CheatSheet_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/EditorView.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_EditorView_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/NotesMap.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_NotesMap_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/TextDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_TextDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/TextEditor.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_TextEditor_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/textEditor/TextToolBar.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_textEditor_TextToolBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/todoEditor/TodoDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_todoEditor_TodoDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/org/kde/klevernotes/contents/ui/todoEditor/ToDoView.qml"), &QmlCacheGeneratedCode::_qt_qml_org_kde_klevernotes_contents_ui_todoEditor_ToDoView_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.structVersion = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_qmlcache_klevernotes_static)() {
    ::unitRegistry();
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_qmlcache_klevernotes_static))
int QT_MANGLE_NAMESPACE(qCleanupResources_qmlcache_klevernotes_static)() {
    return 1;
}
