/****************************************************************************
** Meta object code from reading C++ file 'editorHandler.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../klevernotes/src/logic/editor/editorHandler.hpp"
#include <QtGui/qtextcursor.h>
#include <QtNetwork/QSslPreSharedKeyAuthenticator>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QList>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'editorHandler.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8MdEditor13EditorHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto MdEditor::EditorHandler::qt_create_metaobjectdata<qt_meta_tag_ZN8MdEditor13EditorHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MdEditor::EditorHandler",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "documentChanged",
        "",
        "cursorPositionChanged",
        "position",
        "selectionStartChanged",
        "selectionEndChanged",
        "askForParsing",
        "md",
        "notePath",
        "noteName",
        "counter",
        "renderingFinished",
        "content",
        "surroundingDelimsChanged",
        "QList<int>",
        "delimsTypes",
        "focusEditor",
        "uncheckAction",
        "delimType",
        "newLinkedNotesInfos",
        "QSet<QStringList>",
        "linkedNotesInfos",
        "editorFontChanged",
        "codeHighlightEnabledChanged",
        "newHighlightStyle",
        "pumlEnabledChanged",
        "pumlDarkChanged",
        "editorHighlightEnabledChanged",
        "adaptiveTagSizeChanged",
        "tagScaleChanged",
        "cursorMovedTimeOut",
        "renderPreviewStateChanged",
        "onParsingDone",
        "QSharedPointer<MD::Document>",
        "changeStyles",
        "styles",
        "handleDelims",
        "addDelims",
        "handleTabPressed",
        "backtab",
        "handleReturnPressed",
        "modifier",
        "changeRenderPreviewState",
        "enabled",
        "document",
        "QQuickTextDocument*",
        "cursorPosition",
        "selectionStart",
        "selectionEnd"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'documentChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorPositionChanged'
        QtMocHelpers::SignalData<void(const int)>(7, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'selectionStartChanged'
        QtMocHelpers::SignalData<void(const int)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'selectionEndChanged'
        QtMocHelpers::SignalData<void(const int)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 },
        }}),
        // Signal 'askForParsing'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, unsigned long long int)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 }, { QMetaType::QString, 13 }, { QMetaType::QString, 14 }, { QMetaType::ULongLong, 15 },
        }}),
        // Signal 'renderingFinished'
        QtMocHelpers::SignalData<void(const QString &)>(16, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'surroundingDelimsChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(18, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'focusEditor'
        QtMocHelpers::SignalData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'uncheckAction'
        QtMocHelpers::SignalData<void(const int)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Signal 'newLinkedNotesInfos'
        QtMocHelpers::SignalData<void(const QSet<QStringList> &)>(24, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'editorFontChanged'
        QtMocHelpers::SlotData<void()>(27, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'codeHighlightEnabledChanged'
        QtMocHelpers::SlotData<void()>(28, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'newHighlightStyle'
        QtMocHelpers::SlotData<void()>(29, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pumlEnabledChanged'
        QtMocHelpers::SlotData<void()>(30, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pumlDarkChanged'
        QtMocHelpers::SlotData<void()>(31, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'editorHighlightEnabledChanged'
        QtMocHelpers::SlotData<void()>(32, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'adaptiveTagSizeChanged'
        QtMocHelpers::SlotData<void()>(33, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'tagScaleChanged'
        QtMocHelpers::SlotData<void()>(34, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cursorMovedTimeOut'
        QtMocHelpers::SlotData<void()>(35, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'renderPreviewStateChanged'
        QtMocHelpers::SlotData<void()>(36, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onParsingDone'
        QtMocHelpers::SlotData<void(QSharedPointer<MD::Document>, unsigned long long int)>(37, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 38, 6 }, { QMetaType::ULongLong, 6 },
        }}),
        // Method 'changeStyles'
        QtMocHelpers::MethodData<void(const QStringList &)>(39, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 40 },
        }}),
        // Method 'handleDelims'
        QtMocHelpers::MethodData<void(const bool, const int)>(41, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 42 }, { QMetaType::Int, 23 },
        }}),
        // Method 'handleTabPressed'
        QtMocHelpers::MethodData<void(const bool)>(43, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 44 },
        }}),
        // Method 'handleReturnPressed'
        QtMocHelpers::MethodData<void(const int)>(45, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 46 },
        }}),
        // Method 'changeRenderPreviewState'
        QtMocHelpers::MethodData<void(const bool)>(47, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 48 },
        }}),
        // Method 'changeRenderPreviewState'
        QtMocHelpers::MethodData<void()>(47, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'document'
        QtMocHelpers::PropertyData<QQuickTextDocument*>(49, 0x80000000 | 50, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'cursorPosition'
        QtMocHelpers::PropertyData<int>(51, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'selectionStart'
        QtMocHelpers::PropertyData<int>(52, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'selectionEnd'
        QtMocHelpers::PropertyData<int>(53, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'notePath'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<EditorHandler, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MdEditor::EditorHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8MdEditor13EditorHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8MdEditor13EditorHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8MdEditor13EditorHandlerE_t>.metaTypes,
    nullptr
} };

void MdEditor::EditorHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<EditorHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->documentChanged(); break;
        case 1: _t->cursorPositionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->selectionStartChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->selectionEndChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->askForParsing((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[4]))); break;
        case 5: _t->renderingFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->surroundingDelimsChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 7: _t->focusEditor(); break;
        case 8: _t->uncheckAction((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->newLinkedNotesInfos((*reinterpret_cast<std::add_pointer_t<QSet<QStringList>>>(_a[1]))); break;
        case 10: _t->editorFontChanged(); break;
        case 11: _t->codeHighlightEnabledChanged(); break;
        case 12: _t->newHighlightStyle(); break;
        case 13: _t->pumlEnabledChanged(); break;
        case 14: _t->pumlDarkChanged(); break;
        case 15: _t->editorHighlightEnabledChanged(); break;
        case 16: _t->adaptiveTagSizeChanged(); break;
        case 17: _t->tagScaleChanged(); break;
        case 18: _t->cursorMovedTimeOut(); break;
        case 19: _t->renderPreviewStateChanged(); break;
        case 20: _t->onParsingDone((*reinterpret_cast<std::add_pointer_t<QSharedPointer<MD::Document>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[2]))); break;
        case 21: _t->changeStyles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 22: _t->handleDelims((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 23: _t->handleTabPressed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->handleReturnPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->changeRenderPreviewState((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->changeRenderPreviewState(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QStringList> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)()>(_a, &EditorHandler::documentChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const int )>(_a, &EditorHandler::cursorPositionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const int )>(_a, &EditorHandler::selectionStartChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const int )>(_a, &EditorHandler::selectionEndChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const QString & , const QString & , const QString & , unsigned long long int )>(_a, &EditorHandler::askForParsing, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const QString & )>(_a, &EditorHandler::renderingFinished, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const QList<int> & )>(_a, &EditorHandler::surroundingDelimsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)()>(_a, &EditorHandler::focusEditor, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const int )>(_a, &EditorHandler::uncheckAction, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (EditorHandler::*)(const QSet<QStringList> & )>(_a, &EditorHandler::newLinkedNotesInfos, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickTextDocument* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQuickTextDocument**>(_v) = _t->qQuickDocument(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->cursorPosition(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->selectionStart(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->selectionEnd(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getNotePath(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDocument(*reinterpret_cast<QQuickTextDocument**>(_v)); break;
        case 1: _t->setCursorPosition(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setSelectionStart(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setSelectionEnd(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setNotePath(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MdEditor::EditorHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MdEditor::EditorHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8MdEditor13EditorHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MdEditor::EditorHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void MdEditor::EditorHandler::documentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MdEditor::EditorHandler::cursorPositionChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MdEditor::EditorHandler::selectionStartChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MdEditor::EditorHandler::selectionEndChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MdEditor::EditorHandler::askForParsing(const QString & _t1, const QString & _t2, const QString & _t3, unsigned long long int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void MdEditor::EditorHandler::renderingFinished(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MdEditor::EditorHandler::surroundingDelimsChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void MdEditor::EditorHandler::focusEditor()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MdEditor::EditorHandler::uncheckAction(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void MdEditor::EditorHandler::newLinkedNotesInfos(const QSet<QStringList> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}
QT_WARNING_POP
