/****************************************************************************
** Meta object code from reading C++ file 'kleverconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../kleverconfig.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kleverconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KleverConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto KleverConfig::qt_create_metaobjectdata<qt_meta_tag_ZN12KleverConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KleverConfig",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "storagePathChanged",
        "",
        "defaultFolderNameChanged",
        "defaultNoteNameChanged",
        "editorVisibleChanged",
        "previewVisibleChanged",
        "useSpaceForTabChanged",
        "spacesForTabChanged",
        "visibleToolsChanged",
        "invisibleToolsChanged",
        "sortByLastModifiedChanged",
        "stylePathChanged",
        "codeFontChanged",
        "viewFontChanged",
        "viewBodyColorChanged",
        "viewTextColorChanged",
        "viewTitleColorChanged",
        "viewLinkColorChanged",
        "viewVisitedLinkColorChanged",
        "viewCodeColorChanged",
        "viewHighlightColorChanged",
        "editorFontChanged",
        "editorHighlightEnabledChanged",
        "adaptiveTagSizeEnabledChanged",
        "tagSizeScaleChanged",
        "pumlEnabledChanged",
        "pumlDarkChanged",
        "quickEmojiEnabledChanged",
        "emojiToneChanged",
        "quickEmojiDialogEnabledChanged",
        "codeSynthaxHighlightEnabledChanged",
        "codeSynthaxHighlighterChanged",
        "codeSynthaxHighlighterStyleChanged",
        "noteMapEnabledChanged",
        "storagePath",
        "isStoragePathImmutable",
        "defaultStoragePathValue",
        "defaultFolderName",
        "isDefaultFolderNameImmutable",
        "defaultNoteName",
        "isDefaultNoteNameImmutable",
        "editorVisible",
        "isEditorVisibleImmutable",
        "defaultEditorVisibleValue",
        "previewVisible",
        "isPreviewVisibleImmutable",
        "defaultPreviewVisibleValue",
        "useSpaceForTab",
        "isUseSpaceForTabImmutable",
        "defaultUseSpaceForTabValue",
        "spacesForTab",
        "isSpacesForTabImmutable",
        "defaultSpacesForTabValue",
        "visibleTools",
        "QList<int>",
        "isVisibleToolsImmutable",
        "defaultVisibleToolsValue",
        "invisibleTools",
        "isInvisibleToolsImmutable",
        "defaultInvisibleToolsValue",
        "sortByLastModified",
        "isSortByLastModifiedImmutable",
        "defaultSortByLastModifiedValue",
        "stylePath",
        "isStylePathImmutable",
        "defaultStylePathValue",
        "codeFont",
        "QFont",
        "isCodeFontImmutable",
        "viewFont",
        "isViewFontImmutable",
        "viewBodyColor",
        "isViewBodyColorImmutable",
        "defaultViewBodyColorValue",
        "viewTextColor",
        "isViewTextColorImmutable",
        "defaultViewTextColorValue",
        "viewTitleColor",
        "isViewTitleColorImmutable",
        "defaultViewTitleColorValue",
        "viewLinkColor",
        "isViewLinkColorImmutable",
        "defaultViewLinkColorValue",
        "viewVisitedLinkColor",
        "isViewVisitedLinkColorImmutable",
        "defaultViewVisitedLinkColorValue",
        "viewCodeColor",
        "isViewCodeColorImmutable",
        "defaultViewCodeColorValue",
        "viewHighlightColor",
        "isViewHighlightColorImmutable",
        "defaultViewHighlightColorValue",
        "editorFont",
        "isEditorFontImmutable",
        "editorHighlightEnabled",
        "isEditorHighlightEnabledImmutable",
        "defaultEditorHighlightEnabledValue",
        "adaptiveTagSizeEnabled",
        "isAdaptiveTagSizeEnabledImmutable",
        "defaultAdaptiveTagSizeEnabledValue",
        "tagSizeScale",
        "isTagSizeScaleImmutable",
        "defaultTagSizeScaleValue",
        "pumlEnabled",
        "isPumlEnabledImmutable",
        "defaultPumlEnabledValue",
        "pumlDark",
        "isPumlDarkImmutable",
        "defaultPumlDarkValue",
        "quickEmojiEnabled",
        "isQuickEmojiEnabledImmutable",
        "defaultQuickEmojiEnabledValue",
        "emojiTone",
        "isEmojiToneImmutable",
        "defaultEmojiToneValue",
        "quickEmojiDialogEnabled",
        "isQuickEmojiDialogEnabledImmutable",
        "defaultQuickEmojiDialogEnabledValue",
        "codeSynthaxHighlightEnabled",
        "isCodeSynthaxHighlightEnabledImmutable",
        "defaultCodeSynthaxHighlightEnabledValue",
        "codeSynthaxHighlighter",
        "isCodeSynthaxHighlighterImmutable",
        "codeSynthaxHighlighterStyle",
        "isCodeSynthaxHighlighterStyleImmutable",
        "noteMapEnabled",
        "isNoteMapEnabledImmutable",
        "defaultNoteMapEnabledValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'storagePathChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultFolderNameChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultNoteNameChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'editorVisibleChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previewVisibleChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'useSpaceForTabChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'spacesForTabChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibleToolsChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'invisibleToolsChanged'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sortByLastModifiedChanged'
        QtMocHelpers::SignalData<void()>(15, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stylePathChanged'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'codeFontChanged'
        QtMocHelpers::SignalData<void()>(17, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewFontChanged'
        QtMocHelpers::SignalData<void()>(18, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewBodyColorChanged'
        QtMocHelpers::SignalData<void()>(19, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewTextColorChanged'
        QtMocHelpers::SignalData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewTitleColorChanged'
        QtMocHelpers::SignalData<void()>(21, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewLinkColorChanged'
        QtMocHelpers::SignalData<void()>(22, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewVisitedLinkColorChanged'
        QtMocHelpers::SignalData<void()>(23, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewCodeColorChanged'
        QtMocHelpers::SignalData<void()>(24, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewHighlightColorChanged'
        QtMocHelpers::SignalData<void()>(25, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'editorFontChanged'
        QtMocHelpers::SignalData<void()>(26, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'editorHighlightEnabledChanged'
        QtMocHelpers::SignalData<void()>(27, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'adaptiveTagSizeEnabledChanged'
        QtMocHelpers::SignalData<void()>(28, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tagSizeScaleChanged'
        QtMocHelpers::SignalData<void()>(29, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pumlEnabledChanged'
        QtMocHelpers::SignalData<void()>(30, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pumlDarkChanged'
        QtMocHelpers::SignalData<void()>(31, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'quickEmojiEnabledChanged'
        QtMocHelpers::SignalData<void()>(32, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'emojiToneChanged'
        QtMocHelpers::SignalData<void()>(33, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'quickEmojiDialogEnabledChanged'
        QtMocHelpers::SignalData<void()>(34, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'codeSynthaxHighlightEnabledChanged'
        QtMocHelpers::SignalData<void()>(35, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'codeSynthaxHighlighterChanged'
        QtMocHelpers::SignalData<void()>(36, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'codeSynthaxHighlighterStyleChanged'
        QtMocHelpers::SignalData<void()>(37, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noteMapEnabledChanged'
        QtMocHelpers::SignalData<void()>(38, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'storagePath'
        QtMocHelpers::PropertyData<QString>(39, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'isStoragePathImmutable'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultStoragePathValue'
        QtMocHelpers::PropertyData<QString>(41, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultFolderName'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'isDefaultFolderNameImmutable'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultNoteName'
        QtMocHelpers::PropertyData<QString>(44, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'isDefaultNoteNameImmutable'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'editorVisible'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'isEditorVisibleImmutable'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultEditorVisibleValue'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'previewVisible'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'isPreviewVisibleImmutable'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultPreviewVisibleValue'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'useSpaceForTab'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'isUseSpaceForTabImmutable'
        QtMocHelpers::PropertyData<bool>(53, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultUseSpaceForTabValue'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'spacesForTab'
        QtMocHelpers::PropertyData<int>(55, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'isSpacesForTabImmutable'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultSpacesForTabValue'
        QtMocHelpers::PropertyData<int>(57, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'visibleTools'
        QtMocHelpers::PropertyData<QList<int>>(58, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'isVisibleToolsImmutable'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultVisibleToolsValue'
        QtMocHelpers::PropertyData<QList<int>>(61, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'invisibleTools'
        QtMocHelpers::PropertyData<QList<int>>(62, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 8),
        // property 'isInvisibleToolsImmutable'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultInvisibleToolsValue'
        QtMocHelpers::PropertyData<QList<int>>(64, 0x80000000 | 59, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'sortByLastModified'
        QtMocHelpers::PropertyData<bool>(65, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'isSortByLastModifiedImmutable'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultSortByLastModifiedValue'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'stylePath'
        QtMocHelpers::PropertyData<QString>(68, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'isStylePathImmutable'
        QtMocHelpers::PropertyData<bool>(69, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultStylePathValue'
        QtMocHelpers::PropertyData<QString>(70, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'codeFont'
        QtMocHelpers::PropertyData<QFont>(71, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'isCodeFontImmutable'
        QtMocHelpers::PropertyData<bool>(73, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewFont'
        QtMocHelpers::PropertyData<QFont>(74, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'isViewFontImmutable'
        QtMocHelpers::PropertyData<bool>(75, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewBodyColor'
        QtMocHelpers::PropertyData<QString>(76, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'isViewBodyColorImmutable'
        QtMocHelpers::PropertyData<bool>(77, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultViewBodyColorValue'
        QtMocHelpers::PropertyData<QString>(78, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewTextColor'
        QtMocHelpers::PropertyData<QString>(79, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'isViewTextColorImmutable'
        QtMocHelpers::PropertyData<bool>(80, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultViewTextColorValue'
        QtMocHelpers::PropertyData<QString>(81, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewTitleColor'
        QtMocHelpers::PropertyData<QString>(82, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'isViewTitleColorImmutable'
        QtMocHelpers::PropertyData<bool>(83, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultViewTitleColorValue'
        QtMocHelpers::PropertyData<QString>(84, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewLinkColor'
        QtMocHelpers::PropertyData<QString>(85, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'isViewLinkColorImmutable'
        QtMocHelpers::PropertyData<bool>(86, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultViewLinkColorValue'
        QtMocHelpers::PropertyData<QString>(87, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewVisitedLinkColor'
        QtMocHelpers::PropertyData<QString>(88, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'isViewVisitedLinkColorImmutable'
        QtMocHelpers::PropertyData<bool>(89, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultViewVisitedLinkColorValue'
        QtMocHelpers::PropertyData<QString>(90, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewCodeColor'
        QtMocHelpers::PropertyData<QString>(91, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'isViewCodeColorImmutable'
        QtMocHelpers::PropertyData<bool>(92, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultViewCodeColorValue'
        QtMocHelpers::PropertyData<QString>(93, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewHighlightColor'
        QtMocHelpers::PropertyData<QString>(94, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'isViewHighlightColorImmutable'
        QtMocHelpers::PropertyData<bool>(95, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultViewHighlightColorValue'
        QtMocHelpers::PropertyData<QString>(96, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'editorFont'
        QtMocHelpers::PropertyData<QFont>(97, 0x80000000 | 72, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 20),
        // property 'isEditorFontImmutable'
        QtMocHelpers::PropertyData<bool>(98, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'editorHighlightEnabled'
        QtMocHelpers::PropertyData<bool>(99, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'isEditorHighlightEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(100, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultEditorHighlightEnabledValue'
        QtMocHelpers::PropertyData<bool>(101, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'adaptiveTagSizeEnabled'
        QtMocHelpers::PropertyData<bool>(102, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 22),
        // property 'isAdaptiveTagSizeEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(103, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultAdaptiveTagSizeEnabledValue'
        QtMocHelpers::PropertyData<bool>(104, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tagSizeScale'
        QtMocHelpers::PropertyData<int>(105, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'isTagSizeScaleImmutable'
        QtMocHelpers::PropertyData<bool>(106, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultTagSizeScaleValue'
        QtMocHelpers::PropertyData<int>(107, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'pumlEnabled'
        QtMocHelpers::PropertyData<bool>(108, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
        // property 'isPumlEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(109, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultPumlEnabledValue'
        QtMocHelpers::PropertyData<bool>(110, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'pumlDark'
        QtMocHelpers::PropertyData<bool>(111, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 25),
        // property 'isPumlDarkImmutable'
        QtMocHelpers::PropertyData<bool>(112, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultPumlDarkValue'
        QtMocHelpers::PropertyData<bool>(113, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'quickEmojiEnabled'
        QtMocHelpers::PropertyData<bool>(114, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 26),
        // property 'isQuickEmojiEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(115, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultQuickEmojiEnabledValue'
        QtMocHelpers::PropertyData<bool>(116, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'emojiTone'
        QtMocHelpers::PropertyData<QString>(117, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 27),
        // property 'isEmojiToneImmutable'
        QtMocHelpers::PropertyData<bool>(118, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultEmojiToneValue'
        QtMocHelpers::PropertyData<QString>(119, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'quickEmojiDialogEnabled'
        QtMocHelpers::PropertyData<bool>(120, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 28),
        // property 'isQuickEmojiDialogEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(121, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultQuickEmojiDialogEnabledValue'
        QtMocHelpers::PropertyData<bool>(122, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'codeSynthaxHighlightEnabled'
        QtMocHelpers::PropertyData<bool>(123, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 29),
        // property 'isCodeSynthaxHighlightEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(124, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultCodeSynthaxHighlightEnabledValue'
        QtMocHelpers::PropertyData<bool>(125, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'codeSynthaxHighlighter'
        QtMocHelpers::PropertyData<QString>(126, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 30),
        // property 'isCodeSynthaxHighlighterImmutable'
        QtMocHelpers::PropertyData<bool>(127, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'codeSynthaxHighlighterStyle'
        QtMocHelpers::PropertyData<QString>(128, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 31),
        // property 'isCodeSynthaxHighlighterStyleImmutable'
        QtMocHelpers::PropertyData<bool>(129, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'noteMapEnabled'
        QtMocHelpers::PropertyData<bool>(130, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 32),
        // property 'isNoteMapEnabledImmutable'
        QtMocHelpers::PropertyData<bool>(131, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'defaultNoteMapEnabledValue'
        QtMocHelpers::PropertyData<bool>(132, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<KleverConfig, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KleverConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<KConfigSkeleton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KleverConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KleverConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KleverConfigE_t>.metaTypes,
    nullptr
} };

void KleverConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KleverConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->storagePathChanged(); break;
        case 1: _t->defaultFolderNameChanged(); break;
        case 2: _t->defaultNoteNameChanged(); break;
        case 3: _t->editorVisibleChanged(); break;
        case 4: _t->previewVisibleChanged(); break;
        case 5: _t->useSpaceForTabChanged(); break;
        case 6: _t->spacesForTabChanged(); break;
        case 7: _t->visibleToolsChanged(); break;
        case 8: _t->invisibleToolsChanged(); break;
        case 9: _t->sortByLastModifiedChanged(); break;
        case 10: _t->stylePathChanged(); break;
        case 11: _t->codeFontChanged(); break;
        case 12: _t->viewFontChanged(); break;
        case 13: _t->viewBodyColorChanged(); break;
        case 14: _t->viewTextColorChanged(); break;
        case 15: _t->viewTitleColorChanged(); break;
        case 16: _t->viewLinkColorChanged(); break;
        case 17: _t->viewVisitedLinkColorChanged(); break;
        case 18: _t->viewCodeColorChanged(); break;
        case 19: _t->viewHighlightColorChanged(); break;
        case 20: _t->editorFontChanged(); break;
        case 21: _t->editorHighlightEnabledChanged(); break;
        case 22: _t->adaptiveTagSizeEnabledChanged(); break;
        case 23: _t->tagSizeScaleChanged(); break;
        case 24: _t->pumlEnabledChanged(); break;
        case 25: _t->pumlDarkChanged(); break;
        case 26: _t->quickEmojiEnabledChanged(); break;
        case 27: _t->emojiToneChanged(); break;
        case 28: _t->quickEmojiDialogEnabledChanged(); break;
        case 29: _t->codeSynthaxHighlightEnabledChanged(); break;
        case 30: _t->codeSynthaxHighlighterChanged(); break;
        case 31: _t->codeSynthaxHighlighterStyleChanged(); break;
        case 32: _t->noteMapEnabledChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::storagePathChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::defaultFolderNameChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::defaultNoteNameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::editorVisibleChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::previewVisibleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::useSpaceForTabChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::spacesForTabChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::visibleToolsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::invisibleToolsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::sortByLastModifiedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::stylePathChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::codeFontChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewFontChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewBodyColorChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewTextColorChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewTitleColorChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewLinkColorChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewVisitedLinkColorChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewCodeColorChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::viewHighlightColorChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::editorFontChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::editorHighlightEnabledChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::adaptiveTagSizeEnabledChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::tagSizeScaleChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::pumlEnabledChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::pumlDarkChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::quickEmojiEnabledChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::emojiToneChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::quickEmojiDialogEnabledChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::codeSynthaxHighlightEnabledChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::codeSynthaxHighlighterChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::codeSynthaxHighlighterStyleChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleverConfig::*)()>(_a, &KleverConfig::noteMapEnabledChanged, 32))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 24:
        case 22:
        case 21:
        case 19:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->storagePath(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isStoragePathImmutable(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->defaultStoragePathValue(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->defaultFolderName(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isDefaultFolderNameImmutable(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->defaultNoteName(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isDefaultNoteNameImmutable(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->editorVisible(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->isEditorVisibleImmutable(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->defaultEditorVisibleValue(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->previewVisible(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->isPreviewVisibleImmutable(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->defaultPreviewVisibleValue(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->useSpaceForTab(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->isUseSpaceForTabImmutable(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->defaultUseSpaceForTabValue(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->spacesForTab(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->isSpacesForTabImmutable(); break;
        case 18: *reinterpret_cast<int*>(_v) = _t->defaultSpacesForTabValue(); break;
        case 19: *reinterpret_cast<QList<int>*>(_v) = _t->visibleTools(); break;
        case 20: *reinterpret_cast<bool*>(_v) = _t->isVisibleToolsImmutable(); break;
        case 21: *reinterpret_cast<QList<int>*>(_v) = _t->defaultVisibleToolsValue(); break;
        case 22: *reinterpret_cast<QList<int>*>(_v) = _t->invisibleTools(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->isInvisibleToolsImmutable(); break;
        case 24: *reinterpret_cast<QList<int>*>(_v) = _t->defaultInvisibleToolsValue(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->sortByLastModified(); break;
        case 26: *reinterpret_cast<bool*>(_v) = _t->isSortByLastModifiedImmutable(); break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->defaultSortByLastModifiedValue(); break;
        case 28: *reinterpret_cast<QString*>(_v) = _t->stylePath(); break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->isStylePathImmutable(); break;
        case 30: *reinterpret_cast<QString*>(_v) = _t->defaultStylePathValue(); break;
        case 31: *reinterpret_cast<QFont*>(_v) = _t->codeFont(); break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->isCodeFontImmutable(); break;
        case 33: *reinterpret_cast<QFont*>(_v) = _t->viewFont(); break;
        case 34: *reinterpret_cast<bool*>(_v) = _t->isViewFontImmutable(); break;
        case 35: *reinterpret_cast<QString*>(_v) = _t->viewBodyColor(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->isViewBodyColorImmutable(); break;
        case 37: *reinterpret_cast<QString*>(_v) = _t->defaultViewBodyColorValue(); break;
        case 38: *reinterpret_cast<QString*>(_v) = _t->viewTextColor(); break;
        case 39: *reinterpret_cast<bool*>(_v) = _t->isViewTextColorImmutable(); break;
        case 40: *reinterpret_cast<QString*>(_v) = _t->defaultViewTextColorValue(); break;
        case 41: *reinterpret_cast<QString*>(_v) = _t->viewTitleColor(); break;
        case 42: *reinterpret_cast<bool*>(_v) = _t->isViewTitleColorImmutable(); break;
        case 43: *reinterpret_cast<QString*>(_v) = _t->defaultViewTitleColorValue(); break;
        case 44: *reinterpret_cast<QString*>(_v) = _t->viewLinkColor(); break;
        case 45: *reinterpret_cast<bool*>(_v) = _t->isViewLinkColorImmutable(); break;
        case 46: *reinterpret_cast<QString*>(_v) = _t->defaultViewLinkColorValue(); break;
        case 47: *reinterpret_cast<QString*>(_v) = _t->viewVisitedLinkColor(); break;
        case 48: *reinterpret_cast<bool*>(_v) = _t->isViewVisitedLinkColorImmutable(); break;
        case 49: *reinterpret_cast<QString*>(_v) = _t->defaultViewVisitedLinkColorValue(); break;
        case 50: *reinterpret_cast<QString*>(_v) = _t->viewCodeColor(); break;
        case 51: *reinterpret_cast<bool*>(_v) = _t->isViewCodeColorImmutable(); break;
        case 52: *reinterpret_cast<QString*>(_v) = _t->defaultViewCodeColorValue(); break;
        case 53: *reinterpret_cast<QString*>(_v) = _t->viewHighlightColor(); break;
        case 54: *reinterpret_cast<bool*>(_v) = _t->isViewHighlightColorImmutable(); break;
        case 55: *reinterpret_cast<QString*>(_v) = _t->defaultViewHighlightColorValue(); break;
        case 56: *reinterpret_cast<QFont*>(_v) = _t->editorFont(); break;
        case 57: *reinterpret_cast<bool*>(_v) = _t->isEditorFontImmutable(); break;
        case 58: *reinterpret_cast<bool*>(_v) = _t->editorHighlightEnabled(); break;
        case 59: *reinterpret_cast<bool*>(_v) = _t->isEditorHighlightEnabledImmutable(); break;
        case 60: *reinterpret_cast<bool*>(_v) = _t->defaultEditorHighlightEnabledValue(); break;
        case 61: *reinterpret_cast<bool*>(_v) = _t->adaptiveTagSizeEnabled(); break;
        case 62: *reinterpret_cast<bool*>(_v) = _t->isAdaptiveTagSizeEnabledImmutable(); break;
        case 63: *reinterpret_cast<bool*>(_v) = _t->defaultAdaptiveTagSizeEnabledValue(); break;
        case 64: *reinterpret_cast<int*>(_v) = _t->tagSizeScale(); break;
        case 65: *reinterpret_cast<bool*>(_v) = _t->isTagSizeScaleImmutable(); break;
        case 66: *reinterpret_cast<int*>(_v) = _t->defaultTagSizeScaleValue(); break;
        case 67: *reinterpret_cast<bool*>(_v) = _t->pumlEnabled(); break;
        case 68: *reinterpret_cast<bool*>(_v) = _t->isPumlEnabledImmutable(); break;
        case 69: *reinterpret_cast<bool*>(_v) = _t->defaultPumlEnabledValue(); break;
        case 70: *reinterpret_cast<bool*>(_v) = _t->pumlDark(); break;
        case 71: *reinterpret_cast<bool*>(_v) = _t->isPumlDarkImmutable(); break;
        case 72: *reinterpret_cast<bool*>(_v) = _t->defaultPumlDarkValue(); break;
        case 73: *reinterpret_cast<bool*>(_v) = _t->quickEmojiEnabled(); break;
        case 74: *reinterpret_cast<bool*>(_v) = _t->isQuickEmojiEnabledImmutable(); break;
        case 75: *reinterpret_cast<bool*>(_v) = _t->defaultQuickEmojiEnabledValue(); break;
        case 76: *reinterpret_cast<QString*>(_v) = _t->emojiTone(); break;
        case 77: *reinterpret_cast<bool*>(_v) = _t->isEmojiToneImmutable(); break;
        case 78: *reinterpret_cast<QString*>(_v) = _t->defaultEmojiToneValue(); break;
        case 79: *reinterpret_cast<bool*>(_v) = _t->quickEmojiDialogEnabled(); break;
        case 80: *reinterpret_cast<bool*>(_v) = _t->isQuickEmojiDialogEnabledImmutable(); break;
        case 81: *reinterpret_cast<bool*>(_v) = _t->defaultQuickEmojiDialogEnabledValue(); break;
        case 82: *reinterpret_cast<bool*>(_v) = _t->codeSynthaxHighlightEnabled(); break;
        case 83: *reinterpret_cast<bool*>(_v) = _t->isCodeSynthaxHighlightEnabledImmutable(); break;
        case 84: *reinterpret_cast<bool*>(_v) = _t->defaultCodeSynthaxHighlightEnabledValue(); break;
        case 85: *reinterpret_cast<QString*>(_v) = _t->codeSynthaxHighlighter(); break;
        case 86: *reinterpret_cast<bool*>(_v) = _t->isCodeSynthaxHighlighterImmutable(); break;
        case 87: *reinterpret_cast<QString*>(_v) = _t->codeSynthaxHighlighterStyle(); break;
        case 88: *reinterpret_cast<bool*>(_v) = _t->isCodeSynthaxHighlighterStyleImmutable(); break;
        case 89: *reinterpret_cast<bool*>(_v) = _t->noteMapEnabled(); break;
        case 90: *reinterpret_cast<bool*>(_v) = _t->isNoteMapEnabledImmutable(); break;
        case 91: *reinterpret_cast<bool*>(_v) = _t->defaultNoteMapEnabledValue(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setStoragePath(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setDefaultFolderName(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setDefaultNoteName(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setEditorVisible(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setPreviewVisible(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setUseSpaceForTab(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->setSpacesForTab(*reinterpret_cast<int*>(_v)); break;
        case 19: _t->setVisibleTools(*reinterpret_cast<QList<int>*>(_v)); break;
        case 22: _t->setInvisibleTools(*reinterpret_cast<QList<int>*>(_v)); break;
        case 25: _t->setSortByLastModified(*reinterpret_cast<bool*>(_v)); break;
        case 28: _t->setStylePath(*reinterpret_cast<QString*>(_v)); break;
        case 31: _t->setCodeFont(*reinterpret_cast<QFont*>(_v)); break;
        case 33: _t->setViewFont(*reinterpret_cast<QFont*>(_v)); break;
        case 35: _t->setViewBodyColor(*reinterpret_cast<QString*>(_v)); break;
        case 38: _t->setViewTextColor(*reinterpret_cast<QString*>(_v)); break;
        case 41: _t->setViewTitleColor(*reinterpret_cast<QString*>(_v)); break;
        case 44: _t->setViewLinkColor(*reinterpret_cast<QString*>(_v)); break;
        case 47: _t->setViewVisitedLinkColor(*reinterpret_cast<QString*>(_v)); break;
        case 50: _t->setViewCodeColor(*reinterpret_cast<QString*>(_v)); break;
        case 53: _t->setViewHighlightColor(*reinterpret_cast<QString*>(_v)); break;
        case 56: _t->setEditorFont(*reinterpret_cast<QFont*>(_v)); break;
        case 58: _t->setEditorHighlightEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 61: _t->setAdaptiveTagSizeEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 64: _t->setTagSizeScale(*reinterpret_cast<int*>(_v)); break;
        case 67: _t->setPumlEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 70: _t->setPumlDark(*reinterpret_cast<bool*>(_v)); break;
        case 73: _t->setQuickEmojiEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 76: _t->setEmojiTone(*reinterpret_cast<QString*>(_v)); break;
        case 79: _t->setQuickEmojiDialogEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 82: _t->setCodeSynthaxHighlightEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 85: _t->setCodeSynthaxHighlighter(*reinterpret_cast<QString*>(_v)); break;
        case 87: _t->setCodeSynthaxHighlighterStyle(*reinterpret_cast<QString*>(_v)); break;
        case 89: _t->setNoteMapEnabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KleverConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KleverConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KleverConfigE_t>.strings))
        return static_cast<void*>(this);
    return KConfigSkeleton::qt_metacast(_clname);
}

int KleverConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KConfigSkeleton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 33;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 92;
    }
    return _id;
}

// SIGNAL 0
void KleverConfig::storagePathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KleverConfig::defaultFolderNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KleverConfig::defaultNoteNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KleverConfig::editorVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KleverConfig::previewVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KleverConfig::useSpaceForTabChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KleverConfig::spacesForTabChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KleverConfig::visibleToolsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KleverConfig::invisibleToolsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KleverConfig::sortByLastModifiedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void KleverConfig::stylePathChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void KleverConfig::codeFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KleverConfig::viewFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void KleverConfig::viewBodyColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void KleverConfig::viewTextColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void KleverConfig::viewTitleColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void KleverConfig::viewLinkColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void KleverConfig::viewVisitedLinkColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void KleverConfig::viewCodeColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void KleverConfig::viewHighlightColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void KleverConfig::editorFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void KleverConfig::editorHighlightEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void KleverConfig::adaptiveTagSizeEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void KleverConfig::tagSizeScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void KleverConfig::pumlEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void KleverConfig::pumlDarkChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void KleverConfig::quickEmojiEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void KleverConfig::emojiToneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void KleverConfig::quickEmojiDialogEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void KleverConfig::codeSynthaxHighlightEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void KleverConfig::codeSynthaxHighlighterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void KleverConfig::codeSynthaxHighlighterStyleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}

// SIGNAL 32
void KleverConfig::noteMapEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}
QT_WARNING_POP
