/****************************************************************************
** Meta object code from reading C++ file 'kleverUtility.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../klevernotes/src/logic/kleverUtility.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kleverUtility.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KleverUtilityE_t {};
} // unnamed namespace

template <> constexpr inline auto KleverUtility::qt_create_metaobjectdata<qt_meta_tag_ZN13KleverUtilityE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KleverUtility",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "getName",
        "",
        "path",
        "getPath",
        "QUrl",
        "url",
        "isEmptyDir",
        "isProperPath",
        "parentPath",
        "name",
        "getParentPath",
        "getImageStoragingPath",
        "noteImagesStoringPath",
        "wantedName",
        "iteration",
        "remove",
        "getCssStylesList",
        "QJsonObject",
        "fontInfo",
        "QFont",
        "font",
        "isFlatpak",
        "checkPaste",
        "tempPath",
        "copyToClipboard",
        "toCopy"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'getName'
        QtMocHelpers::MethodData<QString(const QString &) const>(5, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'getPath'
        QtMocHelpers::MethodData<QString(const QUrl &) const>(8, 6, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'isEmptyDir'
        QtMocHelpers::MethodData<bool(const QString &) const>(11, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'isProperPath'
        QtMocHelpers::MethodData<QString(const QString &, const QString &) const>(12, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 },
        }}),
        // Method 'getParentPath'
        QtMocHelpers::MethodData<QString(const QString &) const>(15, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'getImageStoragingPath'
        QtMocHelpers::MethodData<QString(const QString &, const QString &, int) const>(16, 6, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::Int, 19 },
        }}),
        // Method 'getImageStoragingPath'
        QtMocHelpers::MethodData<QString(const QString &, const QString &) const>(16, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QString, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 },
        }}),
        // Method 'remove'
        QtMocHelpers::MethodData<bool(const QString &) const>(20, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 7 },
        }}),
        // Method 'getCssStylesList'
        QtMocHelpers::MethodData<QJsonObject() const>(21, 6, QMC::AccessPublic, 0x80000000 | 22),
        // Method 'fontInfo'
        QtMocHelpers::MethodData<QJsonObject(const QFont &) const>(23, 6, QMC::AccessPublic, 0x80000000 | 22, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'isFlatpak'
        QtMocHelpers::MethodData<bool() const>(26, 6, QMC::AccessPublic, QMetaType::Bool),
        // Method 'checkPaste'
        QtMocHelpers::MethodData<bool(const QString &) const>(27, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 28 },
        }}),
        // Method 'copyToClipboard'
        QtMocHelpers::MethodData<void(const QString &) const>(29, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<KleverUtility, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject KleverUtility::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KleverUtilityE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KleverUtilityE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KleverUtilityE_t>.metaTypes,
    nullptr
} };

void KleverUtility::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KleverUtility *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->getName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->getPath((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->isEmptyDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->isProperPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 4: { QString _r = _t->getParentPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->getImageStoragingPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->getImageStoragingPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->remove((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { QJsonObject _r = _t->getCssStylesList();
            if (_a[0]) *reinterpret_cast<QJsonObject*>(_a[0]) = std::move(_r); }  break;
        case 9: { QJsonObject _r = _t->fontInfo((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJsonObject*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->isFlatpak();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->checkPaste((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: _t->copyToClipboard((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *KleverUtility::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KleverUtility::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KleverUtilityE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KleverUtility::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}
QT_WARNING_POP
