/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<colorschemer.h>)
#  include <colorschemer.h>
#endif
#if __has_include(<documentHandler.h>)
#  include <documentHandler.h>
#endif
#if __has_include(<editorHandler.hpp>)
#  include <editorHandler.hpp>
#endif
#if __has_include(<emojiModel.h>)
#  include <emojiModel.h>
#endif
#if __has_include(<emoticonFilterModel.h>)
#  include <emoticonFilterModel.h>
#endif
#if __has_include(<highlightHelper.h>)
#  include <highlightHelper.h>
#endif
#if __has_include(<kleverUtility.h>)
#  include <kleverUtility.h>
#endif
#if __has_include(<kleverconfig.h>)
#  include <kleverconfig.h>
#endif
#if __has_include(<noteMapper.h>)
#  include <noteMapper.h>
#endif
#if __has_include(<printingHelper.h>)
#  include <printingHelper.h>
#endif
#if __has_include(<qmlLinker.h>)
#  include <qmlLinker.h>
#endif
#if __has_include(<styleHandler.hpp>)
#  include <styleHandler.hpp>
#endif
#if __has_include(<treeModel.h>)
#  include <treeModel.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_klevernotes()
{
    qmlRegisterModule("org.kde.klevernotes", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<ColorSchemer>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<DocumentHandler>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<EmojiModel>("org.kde.klevernotes", 254);
    qmlRegisterEnum<EmojiModel::RoleNames>("EmojiModel::RoleNames");
    qmlRegisterEnum<EmojiModel::Category>("EmojiModel::Category");
    qmlRegisterTypesAndRevisions<EmoticonFilterModel>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<HighlightHelper>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<KleverConfig>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<KleverUtility>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<MdEditor::EditorHandler>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<NoteMapper>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<NoteTreeModel>("org.kde.klevernotes", 254);
    qmlRegisterEnum<NoteTreeModel::ExtraRoles>("NoteTreeModel::ExtraRoles");
    qmlRegisterTypesAndRevisions<PrintingUtility>("org.kde.klevernotes", 254);
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractListModel *>().id();
    QMetaType::fromType<QAbstractProxyModel *>().id();
    QMetaType::fromType<QSortFilterProxyModel *>().id();
    qmlRegisterTypesAndRevisions<QmlLinker>("org.kde.klevernotes", 254);
    qmlRegisterTypesAndRevisions<StyleHandler>("org.kde.klevernotes", 254);
    QT_WARNING_POP
    qmlRegisterModule("org.kde.klevernotes", 254, 254);
}

static const QQmlModuleRegistration orgkdeklevernotesRegistration("org.kde.klevernotes", qml_register_types_org_kde_klevernotes);
