/****************************************************************************
** Meta object code from reading C++ file 'treeModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../klevernotes/src/logic/treeview/treeModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'treeModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13NoteTreeModelE_t {};
} // unnamed namespace

template <> constexpr inline auto NoteTreeModel::qt_create_metaobjectdata<qt_meta_tag_ZN13NoteTreeModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NoteTreeModel",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "errorOccurred",
        "",
        "errorMessage",
        "forceFocus",
        "QModelIndex",
        "rowModelIndex",
        "moveError",
        "newParentIndex",
        "isNote",
        "rowName",
        "newParentPath",
        "oldStorageStructure",
        "newGlobalPathFound",
        "path",
        "globalPathUpdated",
        "oldPath",
        "newPath",
        "globalPathRemoved",
        "initialGlobalPathsSent",
        "initialGlobalPaths",
        "addRow",
        "parentModelIndex",
        "askForExpand",
        "askForFocus",
        "getNoteModelIndex",
        "notePath",
        "initModel",
        "convert",
        "moveRow",
        "newName",
        "removeFromTree",
        "index",
        "permanent",
        "rename",
        "saveMetaData",
        "setProperties",
        "color",
        "icon",
        "noteMapEnabled",
        "ExtraRoles",
        "PathRole",
        "DirRole",
        "ParentPathRole",
        "NameRole",
        "IconNameRole",
        "ColorRole",
        "IsNote",
        "WantFocusRole",
        "WantExpandRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'forceFocus'
        QtMocHelpers::SignalData<void(const QModelIndex &)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'moveError'
        QtMocHelpers::SignalData<void(const QModelIndex &, const QModelIndex &, bool, const QString &, const QString &)>(11, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 12 }, { QMetaType::Bool, 13 }, { QMetaType::QString, 14 },
            { QMetaType::QString, 15 },
        }}),
        // Signal 'oldStorageStructure'
        QtMocHelpers::SignalData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newGlobalPathFound'
        QtMocHelpers::SignalData<void(const QString &)>(17, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'globalPathUpdated'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::QString, 21 },
        }}),
        // Signal 'globalPathRemoved'
        QtMocHelpers::SignalData<void(const QString &)>(22, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Signal 'initialGlobalPathsSent'
        QtMocHelpers::SignalData<void(const QStringList &)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 24 },
        }}),
        // Method 'addRow'
        QtMocHelpers::MethodData<QModelIndex(const QString &, const bool, const QModelIndex &)>(25, 6, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 14 }, { QMetaType::Bool, 13 }, { 0x80000000 | 9, 26 },
        }}),
        // Method 'addRow'
        QtMocHelpers::MethodData<QModelIndex(const QString &, const bool)>(25, 6, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 9, {{
            { QMetaType::QString, 14 }, { QMetaType::Bool, 13 },
        }}),
        // Method 'askForExpand'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(27, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'askForFocus'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(28, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'getNoteModelIndex'
        QtMocHelpers::MethodData<QModelIndex(const QString &)>(29, 6, QMC::AccessPublic, 0x80000000 | 9, {{
            { QMetaType::QString, 30 },
        }}),
        // Method 'initModel'
        QtMocHelpers::MethodData<void(bool)>(31, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Method 'initModel'
        QtMocHelpers::MethodData<void()>(31, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'moveRow'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QModelIndex &, const QString &)>(33, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 12 }, { QMetaType::QString, 34 },
        }}),
        // Method 'moveRow'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QModelIndex &)>(33, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 12 },
        }}),
        // Method 'moveRow'
        QtMocHelpers::MethodData<void(const QModelIndex &)>(33, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Method 'removeFromTree'
        QtMocHelpers::MethodData<void(const QModelIndex &, const bool)>(35, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 36 }, { QMetaType::Bool, 37 },
        }}),
        // Method 'rename'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QString &)>(38, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 34 },
        }}),
        // Method 'saveMetaData'
        QtMocHelpers::MethodData<void()>(39, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'setProperties'
        QtMocHelpers::MethodData<void(const QModelIndex &, const QString, const QString)>(40, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::QString, 41 }, { QMetaType::QString, 42 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'noteMapEnabled'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::Writable | QMC::Designable | QMC::Scriptable | QMC::Stored | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ExtraRoles'
        QtMocHelpers::EnumData<enum ExtraRoles>(44, 44, QMC::EnumFlags{}).add({
            {   45, ExtraRoles::PathRole },
            {   46, ExtraRoles::DirRole },
            {   47, ExtraRoles::ParentPathRole },
            {   48, ExtraRoles::NameRole },
            {   49, ExtraRoles::IconNameRole },
            {   50, ExtraRoles::ColorRole },
            {   51, ExtraRoles::IsNote },
            {   52, ExtraRoles::WantFocusRole },
            {   53, ExtraRoles::WantExpandRole },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<NoteTreeModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject NoteTreeModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13NoteTreeModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13NoteTreeModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13NoteTreeModelE_t>.metaTypes,
    nullptr
} };

void NoteTreeModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NoteTreeModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->forceFocus((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->moveError((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 3: _t->oldStorageStructure(); break;
        case 4: _t->newGlobalPathFound((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->globalPathUpdated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->globalPathRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->initialGlobalPathsSent((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 8: { QModelIndex _r = _t->addRow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 9: { QModelIndex _r = _t->addRow((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->askForExpand((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->askForFocus((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: { QModelIndex _r = _t->getNoteModelIndex((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->initModel((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->initModel(); break;
        case 15: _t->moveRow((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 16: _t->moveRow((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 17: _t->moveRow((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 18: _t->removeFromTree((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->rename((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 20: _t->saveMetaData(); break;
        case 21: _t->setProperties((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)(const QString & )>(_a, &NoteTreeModel::errorOccurred, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)(const QModelIndex & )>(_a, &NoteTreeModel::forceFocus, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)(const QModelIndex & , const QModelIndex & , bool , const QString & , const QString & )>(_a, &NoteTreeModel::moveError, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)()>(_a, &NoteTreeModel::oldStorageStructure, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)(const QString & )>(_a, &NoteTreeModel::newGlobalPathFound, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)(const QString & , const QString & )>(_a, &NoteTreeModel::globalPathUpdated, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)(const QString & )>(_a, &NoteTreeModel::globalPathRemoved, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (NoteTreeModel::*)(const QStringList & )>(_a, &NoteTreeModel::initialGlobalPathsSent, 7))
            return;
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setNoteMapEnabled(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *NoteTreeModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NoteTreeModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13NoteTreeModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int NoteTreeModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void NoteTreeModel::errorOccurred(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NoteTreeModel::forceFocus(const QModelIndex & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NoteTreeModel::moveError(const QModelIndex & _t1, const QModelIndex & _t2, bool _t3, const QString & _t4, const QString & _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 3
void NoteTreeModel::oldStorageStructure()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void NoteTreeModel::newGlobalPathFound(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void NoteTreeModel::globalPathUpdated(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void NoteTreeModel::globalPathRemoved(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void NoteTreeModel::initialGlobalPathsSent(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
