/****************************************************************************
** Meta object code from reading C++ file 'parser.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../klevernotes/src/logic/parser/parser.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'parser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8MdEditor6ParserE_t {};
} // unnamed namespace

template <> constexpr inline auto MdEditor::Parser::qt_create_metaobjectdata<qt_meta_tag_ZN8MdEditor6ParserE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MdEditor::Parser",
        "newData",
        "",
        "done",
        "QSharedPointer<MD::Document>",
        "mdDoc",
        "parseCount",
        "noteLinkingEnabledChangedSignal",
        "on",
        "quickEmojiEnabledChangedSignal",
        "onData",
        "md",
        "noteDir",
        "noteName",
        "counter",
        "noteLinkingEnabledChanged",
        "quickEmojiEnabledChanged",
        "onParse"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newData'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'done'
        QtMocHelpers::SignalData<void(QSharedPointer<MD::Document>, unsigned long long int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::ULongLong, 6 },
        }}),
        // Signal 'noteLinkingEnabledChangedSignal'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'quickEmojiEnabledChangedSignal'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'onData'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, unsigned long long int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 13 }, { QMetaType::ULongLong, 14 },
        }}),
        // Slot 'noteLinkingEnabledChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'quickEmojiEnabledChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'noteLinkingEnabledChanged'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'quickEmojiEnabledChanged'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'onParse'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Parser, qt_meta_tag_ZN8MdEditor6ParserE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MdEditor::Parser::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8MdEditor6ParserE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8MdEditor6ParserE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8MdEditor6ParserE_t>.metaTypes,
    nullptr
} };

void MdEditor::Parser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Parser *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newData(); break;
        case 1: _t->done((*reinterpret_cast<std::add_pointer_t<QSharedPointer<MD::Document>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[2]))); break;
        case 2: _t->noteLinkingEnabledChangedSignal((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->quickEmojiEnabledChangedSignal((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->onData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qulonglong>>(_a[4]))); break;
        case 5: _t->noteLinkingEnabledChanged(); break;
        case 6: _t->quickEmojiEnabledChanged(); break;
        case 7: _t->noteLinkingEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->quickEmojiEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->onParse(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Parser::*)()>(_a, &Parser::newData, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(QSharedPointer<MD::Document> , unsigned long long int )>(_a, &Parser::done, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(bool )>(_a, &Parser::noteLinkingEnabledChangedSignal, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Parser::*)(bool )>(_a, &Parser::quickEmojiEnabledChangedSignal, 3))
            return;
    }
}

const QMetaObject *MdEditor::Parser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MdEditor::Parser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8MdEditor6ParserE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MdEditor::Parser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void MdEditor::Parser::newData()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MdEditor::Parser::done(QSharedPointer<MD::Document> _t1, unsigned long long int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void MdEditor::Parser::noteLinkingEnabledChangedSignal(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MdEditor::Parser::quickEmojiEnabledChangedSignal(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
