/*
    SPDX-FileCopyrightText: 2026 Igor Mironchik <igor.mironchik@gmail.com>
    SPDX-License-Identifier: MIT
*/

// This file is autogenerated.

#ifndef MD4QT_ENTITIES_MAP_H_INCLUDED
#define MD4QT_ENTITIES_MAP_H_INCLUDED

#include <QHash>
#include <QString>

namespace MD
{

extern const QHash<QString, QString> s_entityMap;

} /* namespace MD */

#endif // MD4QT_ENTITIES_MAP_H_INCLUDED
