// This file is generated by kconfig_compiler_kf6 from kleverconfig.kcfg.
// All changes you do to this file will be lost.

#include "kleverconfig.h"

#include <qglobal.h>
#include <QFile>
#include <QQmlEngine>

class KleverConfigHelper
{
  public:
    KleverConfigHelper() : q(nullptr) {}
    ~KleverConfigHelper() { delete q; q = nullptr; }
    KleverConfigHelper(const KleverConfigHelper&) = delete;
    KleverConfigHelper& operator=(const KleverConfigHelper&) = delete;
    KleverConfig *q;
};
Q_GLOBAL_STATIC(KleverConfigHelper, s_globalKleverConfig)
KleverConfig *KleverConfig::self()
{
  if (!s_globalKleverConfig()->q) {
    new KleverConfig;
    s_globalKleverConfig()->q->read();
  }

  return s_globalKleverConfig()->q;
}

KleverConfig *KleverConfig::create(QQmlEngine *, QJSEngine *)
{
  QQmlEngine::setObjectOwnership(self(), QQmlEngine::CppOwnership);
  return self();
}

KleverConfig::KleverConfig( QObject *parent )
  : KConfigSkeleton( QStringLiteral( "KleverNotesrc" ) )
{
  setParent(parent);
  Q_ASSERT(!s_globalKleverConfig()->q);
  s_globalKleverConfig()->q = this;
  KConfigCompilerSignallingItem::NotifyFunction notifyFunction = static_cast<KConfigCompilerSignallingItem::NotifyFunction>(&KleverConfig::itemChanged);

  setCurrentGroup( QStringLiteral( "General" ) );

  KConfigCompilerSignallingItem  *itemStoragePath;
  KConfigSkeleton::ItemPath  *innerItemStoragePath;
  innerItemStoragePath = new KConfigSkeleton::ItemPath( currentGroup(), QStringLiteral( "storagePath" ), mStoragePath, QStringLiteral( "None" ) );
  itemStoragePath = new KConfigCompilerSignallingItem(innerItemStoragePath, this, notifyFunction, signalStoragePathChanged);
  addItem( itemStoragePath, QStringLiteral( "storagePath" ) );
  KConfigCompilerSignallingItem  *itemDefaultFolderName;
  KConfigSkeleton::ItemString  *innerItemDefaultFolderName;
  innerItemDefaultFolderName = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "defaultFolderName" ), mDefaultFolderName );
  itemDefaultFolderName = new KConfigCompilerSignallingItem(innerItemDefaultFolderName, this, notifyFunction, signalDefaultFolderNameChanged);
  addItem( itemDefaultFolderName, QStringLiteral( "defaultFolderName" ) );
  KConfigCompilerSignallingItem  *itemDefaultNoteName;
  KConfigSkeleton::ItemString  *innerItemDefaultNoteName;
  innerItemDefaultNoteName = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "defaultNoteName" ), mDefaultNoteName );
  itemDefaultNoteName = new KConfigCompilerSignallingItem(innerItemDefaultNoteName, this, notifyFunction, signalDefaultNoteNameChanged);
  addItem( itemDefaultNoteName, QStringLiteral( "defaultNoteName" ) );
  KConfigCompilerSignallingItem  *itemEditorVisible;
  KConfigSkeleton::ItemBool  *innerItemEditorVisible;
  innerItemEditorVisible = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "editorVisible" ), mEditorVisible, true );
  itemEditorVisible = new KConfigCompilerSignallingItem(innerItemEditorVisible, this, notifyFunction, signalEditorVisibleChanged);
  addItem( itemEditorVisible, QStringLiteral( "editorVisible" ) );
  KConfigCompilerSignallingItem  *itemPreviewVisible;
  KConfigSkeleton::ItemBool  *innerItemPreviewVisible;
  innerItemPreviewVisible = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "previewVisible" ), mPreviewVisible, true );
  itemPreviewVisible = new KConfigCompilerSignallingItem(innerItemPreviewVisible, this, notifyFunction, signalPreviewVisibleChanged);
  addItem( itemPreviewVisible, QStringLiteral( "previewVisible" ) );
  KConfigCompilerSignallingItem  *itemUseSpaceForTab;
  KConfigSkeleton::ItemBool  *innerItemUseSpaceForTab;
  innerItemUseSpaceForTab = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "useSpaceForTab" ), mUseSpaceForTab, true );
  itemUseSpaceForTab = new KConfigCompilerSignallingItem(innerItemUseSpaceForTab, this, notifyFunction, signalUseSpaceForTabChanged);
  addItem( itemUseSpaceForTab, QStringLiteral( "useSpaceForTab" ) );
  KConfigCompilerSignallingItem  *itemSpacesForTab;
  KConfigSkeleton::ItemInt  *innerItemSpacesForTab;
  innerItemSpacesForTab = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "spacesForTab" ), mSpacesForTab, 4 );
  itemSpacesForTab = new KConfigCompilerSignallingItem(innerItemSpacesForTab, this, notifyFunction, signalSpacesForTabChanged);
  addItem( itemSpacesForTab, QStringLiteral( "spacesForTab" ) );
  QList<int> defaultvisibleTools;

  KConfigCompilerSignallingItem  *itemVisibleTools;
  KConfigSkeleton::ItemIntList  *innerItemVisibleTools;
  innerItemVisibleTools = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "visibleTools" ), mVisibleTools, defaultvisibleTools );
  itemVisibleTools = new KConfigCompilerSignallingItem(innerItemVisibleTools, this, notifyFunction, signalVisibleToolsChanged);
  addItem( itemVisibleTools, QStringLiteral( "visibleTools" ) );
  QList<int> defaultinvisibleTools;

  KConfigCompilerSignallingItem  *itemInvisibleTools;
  KConfigSkeleton::ItemIntList  *innerItemInvisibleTools;
  innerItemInvisibleTools = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "invisibleTools" ), mInvisibleTools, defaultinvisibleTools );
  itemInvisibleTools = new KConfigCompilerSignallingItem(innerItemInvisibleTools, this, notifyFunction, signalInvisibleToolsChanged);
  addItem( itemInvisibleTools, QStringLiteral( "invisibleTools" ) );
  KConfigCompilerSignallingItem  *itemSortByLastModified;
  KConfigSkeleton::ItemBool  *innerItemSortByLastModified;
  innerItemSortByLastModified = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "sortByLastModified" ), mSortByLastModified, false );
  itemSortByLastModified = new KConfigCompilerSignallingItem(innerItemSortByLastModified, this, notifyFunction, signalSortByLastModifiedChanged);
  addItem( itemSortByLastModified, QStringLiteral( "sortByLastModified" ) );

  setCurrentGroup( QStringLiteral( "Appearance" ) );

  KConfigCompilerSignallingItem  *itemStylePath;
  KConfigSkeleton::ItemPath  *innerItemStylePath;
  innerItemStylePath = new KConfigSkeleton::ItemPath( currentGroup(), QStringLiteral( "stylePath" ), mStylePath, QStringLiteral( ":/KleverStyle.css" ) );
  itemStylePath = new KConfigCompilerSignallingItem(innerItemStylePath, this, notifyFunction, signalStylePathChanged);
  addItem( itemStylePath, QStringLiteral( "stylePath" ) );
  KConfigCompilerSignallingItem  *itemCodeFont;
  KConfigSkeleton::ItemFont  *innerItemCodeFont;
  innerItemCodeFont = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "codeFont" ), mCodeFont );
  itemCodeFont = new KConfigCompilerSignallingItem(innerItemCodeFont, this, notifyFunction, signalCodeFontChanged);
  addItem( itemCodeFont, QStringLiteral( "codeFont" ) );
  KConfigCompilerSignallingItem  *itemViewFont;
  KConfigSkeleton::ItemFont  *innerItemViewFont;
  innerItemViewFont = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "viewFont" ), mViewFont );
  itemViewFont = new KConfigCompilerSignallingItem(innerItemViewFont, this, notifyFunction, signalViewFontChanged);
  addItem( itemViewFont, QStringLiteral( "viewFont" ) );
  KConfigCompilerSignallingItem  *itemViewBodyColor;
  KConfigSkeleton::ItemString  *innerItemViewBodyColor;
  innerItemViewBodyColor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "viewBodyColor" ), mViewBodyColor, QStringLiteral( "None" ) );
  itemViewBodyColor = new KConfigCompilerSignallingItem(innerItemViewBodyColor, this, notifyFunction, signalViewBodyColorChanged);
  addItem( itemViewBodyColor, QStringLiteral( "viewBodyColor" ) );
  KConfigCompilerSignallingItem  *itemViewTextColor;
  KConfigSkeleton::ItemString  *innerItemViewTextColor;
  innerItemViewTextColor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "viewTextColor" ), mViewTextColor, QStringLiteral( "None" ) );
  itemViewTextColor = new KConfigCompilerSignallingItem(innerItemViewTextColor, this, notifyFunction, signalViewTextColorChanged);
  addItem( itemViewTextColor, QStringLiteral( "viewTextColor" ) );
  KConfigCompilerSignallingItem  *itemViewTitleColor;
  KConfigSkeleton::ItemString  *innerItemViewTitleColor;
  innerItemViewTitleColor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "viewTitleColor" ), mViewTitleColor, QStringLiteral( "None" ) );
  itemViewTitleColor = new KConfigCompilerSignallingItem(innerItemViewTitleColor, this, notifyFunction, signalViewTitleColorChanged);
  addItem( itemViewTitleColor, QStringLiteral( "viewTitleColor" ) );
  KConfigCompilerSignallingItem  *itemViewLinkColor;
  KConfigSkeleton::ItemString  *innerItemViewLinkColor;
  innerItemViewLinkColor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "viewLinkColor" ), mViewLinkColor, QStringLiteral( "None" ) );
  itemViewLinkColor = new KConfigCompilerSignallingItem(innerItemViewLinkColor, this, notifyFunction, signalViewLinkColorChanged);
  addItem( itemViewLinkColor, QStringLiteral( "viewLinkColor" ) );
  KConfigCompilerSignallingItem  *itemViewVisitedLinkColor;
  KConfigSkeleton::ItemString  *innerItemViewVisitedLinkColor;
  innerItemViewVisitedLinkColor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "viewVisitedLinkColor" ), mViewVisitedLinkColor, QStringLiteral( "None" ) );
  itemViewVisitedLinkColor = new KConfigCompilerSignallingItem(innerItemViewVisitedLinkColor, this, notifyFunction, signalViewVisitedLinkColorChanged);
  addItem( itemViewVisitedLinkColor, QStringLiteral( "viewVisitedLinkColor" ) );
  KConfigCompilerSignallingItem  *itemViewCodeColor;
  KConfigSkeleton::ItemString  *innerItemViewCodeColor;
  innerItemViewCodeColor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "viewCodeColor" ), mViewCodeColor, QStringLiteral( "None" ) );
  itemViewCodeColor = new KConfigCompilerSignallingItem(innerItemViewCodeColor, this, notifyFunction, signalViewCodeColorChanged);
  addItem( itemViewCodeColor, QStringLiteral( "viewCodeColor" ) );
  KConfigCompilerSignallingItem  *itemViewHighlightColor;
  KConfigSkeleton::ItemString  *innerItemViewHighlightColor;
  innerItemViewHighlightColor = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "viewHighlightColor" ), mViewHighlightColor, QStringLiteral( "None" ) );
  itemViewHighlightColor = new KConfigCompilerSignallingItem(innerItemViewHighlightColor, this, notifyFunction, signalViewHighlightColorChanged);
  addItem( itemViewHighlightColor, QStringLiteral( "viewHighlightColor" ) );

  setCurrentGroup( QStringLiteral( "Editor" ) );

  KConfigCompilerSignallingItem  *itemEditorFont;
  KConfigSkeleton::ItemFont  *innerItemEditorFont;
  innerItemEditorFont = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "editorFont" ), mEditorFont );
  itemEditorFont = new KConfigCompilerSignallingItem(innerItemEditorFont, this, notifyFunction, signalEditorFontChanged);
  addItem( itemEditorFont, QStringLiteral( "editorFont" ) );
  KConfigCompilerSignallingItem  *itemEditorHighlightEnabled;
  KConfigSkeleton::ItemBool  *innerItemEditorHighlightEnabled;
  innerItemEditorHighlightEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "editorHighlightEnabled" ), mEditorHighlightEnabled, true );
  itemEditorHighlightEnabled = new KConfigCompilerSignallingItem(innerItemEditorHighlightEnabled, this, notifyFunction, signalEditorHighlightEnabledChanged);
  addItem( itemEditorHighlightEnabled, QStringLiteral( "editorHighlightEnabled" ) );
  KConfigCompilerSignallingItem  *itemAdaptiveTagSizeEnabled;
  KConfigSkeleton::ItemBool  *innerItemAdaptiveTagSizeEnabled;
  innerItemAdaptiveTagSizeEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "adaptiveTagSizeEnabled" ), mAdaptiveTagSizeEnabled, false );
  itemAdaptiveTagSizeEnabled = new KConfigCompilerSignallingItem(innerItemAdaptiveTagSizeEnabled, this, notifyFunction, signalAdaptiveTagSizeEnabledChanged);
  addItem( itemAdaptiveTagSizeEnabled, QStringLiteral( "adaptiveTagSizeEnabled" ) );
  KConfigCompilerSignallingItem  *itemTagSizeScale;
  KConfigSkeleton::ItemInt  *innerItemTagSizeScale;
  innerItemTagSizeScale = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "tagSizeScale" ), mTagSizeScale, 50 );
  itemTagSizeScale = new KConfigCompilerSignallingItem(innerItemTagSizeScale, this, notifyFunction, signalTagSizeScaleChanged);
  addItem( itemTagSizeScale, QStringLiteral( "tagSizeScale" ) );

  setCurrentGroup( QStringLiteral( "Plugins" ) );

  KConfigCompilerSignallingItem  *itemPumlEnabled;
  KConfigSkeleton::ItemBool  *innerItemPumlEnabled;
  innerItemPumlEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "pumlEnabled" ), mPumlEnabled, false );
  itemPumlEnabled = new KConfigCompilerSignallingItem(innerItemPumlEnabled, this, notifyFunction, signalPumlEnabledChanged);
  addItem( itemPumlEnabled, QStringLiteral( "pumlEnabled" ) );
  KConfigCompilerSignallingItem  *itemPumlDark;
  KConfigSkeleton::ItemBool  *innerItemPumlDark;
  innerItemPumlDark = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "pumlDark" ), mPumlDark, false );
  itemPumlDark = new KConfigCompilerSignallingItem(innerItemPumlDark, this, notifyFunction, signalPumlDarkChanged);
  addItem( itemPumlDark, QStringLiteral( "pumlDark" ) );
  KConfigCompilerSignallingItem  *itemQuickEmojiEnabled;
  KConfigSkeleton::ItemBool  *innerItemQuickEmojiEnabled;
  innerItemQuickEmojiEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "quickEmojiEnabled" ), mQuickEmojiEnabled, false );
  itemQuickEmojiEnabled = new KConfigCompilerSignallingItem(innerItemQuickEmojiEnabled, this, notifyFunction, signalQuickEmojiEnabledChanged);
  addItem( itemQuickEmojiEnabled, QStringLiteral( "quickEmojiEnabled" ) );
  KConfigCompilerSignallingItem  *itemEmojiTone;
  KConfigSkeleton::ItemString  *innerItemEmojiTone;
  innerItemEmojiTone = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "emojiTone" ), mEmojiTone, QStringLiteral( "None" ) );
  itemEmojiTone = new KConfigCompilerSignallingItem(innerItemEmojiTone, this, notifyFunction, signalEmojiToneChanged);
  addItem( itemEmojiTone, QStringLiteral( "emojiTone" ) );
  KConfigCompilerSignallingItem  *itemQuickEmojiDialogEnabled;
  KConfigSkeleton::ItemBool  *innerItemQuickEmojiDialogEnabled;
  innerItemQuickEmojiDialogEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "quickEmojiDialogEnabled" ), mQuickEmojiDialogEnabled, false );
  itemQuickEmojiDialogEnabled = new KConfigCompilerSignallingItem(innerItemQuickEmojiDialogEnabled, this, notifyFunction, signalQuickEmojiDialogEnabledChanged);
  addItem( itemQuickEmojiDialogEnabled, QStringLiteral( "quickEmojiDialogEnabled" ) );
  KConfigCompilerSignallingItem  *itemCodeSynthaxHighlightEnabled;
  KConfigSkeleton::ItemBool  *innerItemCodeSynthaxHighlightEnabled;
  innerItemCodeSynthaxHighlightEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "codeSynthaxHighlightEnabled" ), mCodeSynthaxHighlightEnabled, false );
  itemCodeSynthaxHighlightEnabled = new KConfigCompilerSignallingItem(innerItemCodeSynthaxHighlightEnabled, this, notifyFunction, signalCodeSynthaxHighlightEnabledChanged);
  addItem( itemCodeSynthaxHighlightEnabled, QStringLiteral( "codeSynthaxHighlightEnabled" ) );
  KConfigCompilerSignallingItem  *itemCodeSynthaxHighlighter;
  KConfigSkeleton::ItemString  *innerItemCodeSynthaxHighlighter;
  innerItemCodeSynthaxHighlighter = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "codeSynthaxHighlighter" ), mCodeSynthaxHighlighter );
  itemCodeSynthaxHighlighter = new KConfigCompilerSignallingItem(innerItemCodeSynthaxHighlighter, this, notifyFunction, signalCodeSynthaxHighlighterChanged);
  addItem( itemCodeSynthaxHighlighter, QStringLiteral( "codeSynthaxHighlighter" ) );
  KConfigCompilerSignallingItem  *itemCodeSynthaxHighlighterStyle;
  KConfigSkeleton::ItemString  *innerItemCodeSynthaxHighlighterStyle;
  innerItemCodeSynthaxHighlighterStyle = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "codeSynthaxHighlighterStyle" ), mCodeSynthaxHighlighterStyle );
  itemCodeSynthaxHighlighterStyle = new KConfigCompilerSignallingItem(innerItemCodeSynthaxHighlighterStyle, this, notifyFunction, signalCodeSynthaxHighlighterStyleChanged);
  addItem( itemCodeSynthaxHighlighterStyle, QStringLiteral( "codeSynthaxHighlighterStyle" ) );
  KConfigCompilerSignallingItem  *itemNoteMapEnabled;
  KConfigSkeleton::ItemBool  *innerItemNoteMapEnabled;
  innerItemNoteMapEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "noteMapEnabled" ), mNoteMapEnabled, false );
  itemNoteMapEnabled = new KConfigCompilerSignallingItem(innerItemNoteMapEnabled, this, notifyFunction, signalNoteMapEnabledChanged);
  addItem( itemNoteMapEnabled, QStringLiteral( "noteMapEnabled" ) );
}

QString KleverConfig::defaultStoragePathValue_helper()
{

  return QStringLiteral( "None" );
}

bool KleverConfig::defaultEditorVisibleValue_helper()
{

  return true;
}

bool KleverConfig::defaultPreviewVisibleValue_helper()
{

  return true;
}

bool KleverConfig::defaultUseSpaceForTabValue_helper()
{

  return true;
}

int KleverConfig::defaultSpacesForTabValue_helper()
{

  return 4;
}

QList<int> KleverConfig::defaultVisibleToolsValue_helper()
{
  QList<int> defaultvisibleTools;

  return defaultvisibleTools;
}

QList<int> KleverConfig::defaultInvisibleToolsValue_helper()
{
  QList<int> defaultinvisibleTools;

  return defaultinvisibleTools;
}

bool KleverConfig::defaultSortByLastModifiedValue_helper()
{

  return false;
}

QString KleverConfig::defaultStylePathValue_helper()
{

  return QStringLiteral( ":/KleverStyle.css" );
}

QString KleverConfig::defaultViewBodyColorValue_helper()
{

  return QStringLiteral( "None" );
}

QString KleverConfig::defaultViewTextColorValue_helper()
{

  return QStringLiteral( "None" );
}

QString KleverConfig::defaultViewTitleColorValue_helper()
{

  return QStringLiteral( "None" );
}

QString KleverConfig::defaultViewLinkColorValue_helper()
{

  return QStringLiteral( "None" );
}

QString KleverConfig::defaultViewVisitedLinkColorValue_helper()
{

  return QStringLiteral( "None" );
}

QString KleverConfig::defaultViewCodeColorValue_helper()
{

  return QStringLiteral( "None" );
}

QString KleverConfig::defaultViewHighlightColorValue_helper()
{

  return QStringLiteral( "None" );
}

bool KleverConfig::defaultEditorHighlightEnabledValue_helper()
{

  return true;
}

bool KleverConfig::defaultAdaptiveTagSizeEnabledValue_helper()
{

  return false;
}

int KleverConfig::defaultTagSizeScaleValue_helper()
{

  return 50;
}

bool KleverConfig::defaultPumlEnabledValue_helper()
{

  return false;
}

bool KleverConfig::defaultPumlDarkValue_helper()
{

  return false;
}

bool KleverConfig::defaultQuickEmojiEnabledValue_helper()
{

  return false;
}

QString KleverConfig::defaultEmojiToneValue_helper()
{

  return QStringLiteral( "None" );
}

bool KleverConfig::defaultQuickEmojiDialogEnabledValue_helper()
{

  return false;
}

bool KleverConfig::defaultCodeSynthaxHighlightEnabledValue_helper()
{

  return false;
}

bool KleverConfig::defaultNoteMapEnabledValue_helper()
{

  return false;
}

KleverConfig::~KleverConfig()
{
  if (s_globalKleverConfig.exists() && !s_globalKleverConfig.isDestroyed()) {
    s_globalKleverConfig()->q = nullptr;
  }
}


void KleverConfig::itemChanged(quint64 signalFlag) {

  switch (signalFlag) {
  case signalStoragePathChanged:
    Q_EMIT storagePathChanged();
    break;
  case signalDefaultFolderNameChanged:
    Q_EMIT defaultFolderNameChanged();
    break;
  case signalDefaultNoteNameChanged:
    Q_EMIT defaultNoteNameChanged();
    break;
  case signalEditorVisibleChanged:
    Q_EMIT editorVisibleChanged();
    break;
  case signalPreviewVisibleChanged:
    Q_EMIT previewVisibleChanged();
    break;
  case signalUseSpaceForTabChanged:
    Q_EMIT useSpaceForTabChanged();
    break;
  case signalSpacesForTabChanged:
    Q_EMIT spacesForTabChanged();
    break;
  case signalVisibleToolsChanged:
    Q_EMIT visibleToolsChanged();
    break;
  case signalInvisibleToolsChanged:
    Q_EMIT invisibleToolsChanged();
    break;
  case signalSortByLastModifiedChanged:
    Q_EMIT sortByLastModifiedChanged();
    break;
  case signalStylePathChanged:
    Q_EMIT stylePathChanged();
    break;
  case signalCodeFontChanged:
    Q_EMIT codeFontChanged();
    break;
  case signalViewFontChanged:
    Q_EMIT viewFontChanged();
    break;
  case signalViewBodyColorChanged:
    Q_EMIT viewBodyColorChanged();
    break;
  case signalViewTextColorChanged:
    Q_EMIT viewTextColorChanged();
    break;
  case signalViewTitleColorChanged:
    Q_EMIT viewTitleColorChanged();
    break;
  case signalViewLinkColorChanged:
    Q_EMIT viewLinkColorChanged();
    break;
  case signalViewVisitedLinkColorChanged:
    Q_EMIT viewVisitedLinkColorChanged();
    break;
  case signalViewCodeColorChanged:
    Q_EMIT viewCodeColorChanged();
    break;
  case signalViewHighlightColorChanged:
    Q_EMIT viewHighlightColorChanged();
    break;
  case signalEditorFontChanged:
    Q_EMIT editorFontChanged();
    break;
  case signalEditorHighlightEnabledChanged:
    Q_EMIT editorHighlightEnabledChanged();
    break;
  case signalAdaptiveTagSizeEnabledChanged:
    Q_EMIT adaptiveTagSizeEnabledChanged();
    break;
  case signalTagSizeScaleChanged:
    Q_EMIT tagSizeScaleChanged();
    break;
  case signalPumlEnabledChanged:
    Q_EMIT pumlEnabledChanged();
    break;
  case signalPumlDarkChanged:
    Q_EMIT pumlDarkChanged();
    break;
  case signalQuickEmojiEnabledChanged:
    Q_EMIT quickEmojiEnabledChanged();
    break;
  case signalEmojiToneChanged:
    Q_EMIT emojiToneChanged();
    break;
  case signalQuickEmojiDialogEnabledChanged:
    Q_EMIT quickEmojiDialogEnabledChanged();
    break;
  case signalCodeSynthaxHighlightEnabledChanged:
    Q_EMIT codeSynthaxHighlightEnabledChanged();
    break;
  case signalCodeSynthaxHighlighterChanged:
    Q_EMIT codeSynthaxHighlighterChanged();
    break;
  case signalCodeSynthaxHighlighterStyleChanged:
    Q_EMIT codeSynthaxHighlighterStyleChanged();
    break;
  case signalNoteMapEnabledChanged:
    Q_EMIT noteMapEnabledChanged();
    break;
  }
}
