/****************************************************************************
** Meta object code from reading C++ file 'noteMapper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../klevernotes/src/logic/parser/plugins/noteMapper/noteMapper.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'noteMapper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10NoteMapperE_t {};
} // unnamed namespace

template <> constexpr inline auto NoteMapper::qt_create_metaobjectdata<qt_meta_tag_ZN10NoteMapperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NoteMapper",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "getCleanedHeaderAndLevel",
        "QVariantList",
        "",
        "header",
        "saveMap",
        "getNoteHeaders",
        "QList<QVariantMap>",
        "notePath",
        "addInitialGlobalPaths",
        "paths",
        "addGlobalPath",
        "path",
        "updateGlobalPath",
        "_oldPath",
        "_newPath",
        "removeGlobalPath",
        "_path",
        "addLinkedNotesInfos",
        "QList<QStringList>",
        "linkedNotesInfos",
        "updatePathInfo",
        "headers",
        "count"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'getCleanedHeaderAndLevel'
        QtMocHelpers::MethodData<QVariantList(const QString &) const>(5, 7, QMC::AccessPublic, 0x80000000 | 6, {{
            { QMetaType::QString, 8 },
        }}),
        // Method 'saveMap'
        QtMocHelpers::MethodData<void() const>(9, 7, QMC::AccessPublic, QMetaType::Void),
        // Method 'getNoteHeaders'
        QtMocHelpers::MethodData<QList<QVariantMap>(const QString &)>(10, 7, QMC::AccessPublic, 0x80000000 | 11, {{
            { QMetaType::QString, 12 },
        }}),
        // Method 'addInitialGlobalPaths'
        QtMocHelpers::MethodData<void(const QStringList &)>(13, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 },
        }}),
        // Method 'addGlobalPath'
        QtMocHelpers::MethodData<void(const QString &)>(15, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Method 'updateGlobalPath'
        QtMocHelpers::MethodData<void(const QString &, const QString &)>(17, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 }, { QMetaType::QString, 19 },
        }}),
        // Method 'removeGlobalPath'
        QtMocHelpers::MethodData<void(const QString &)>(20, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'addLinkedNotesInfos'
        QtMocHelpers::MethodData<void(const QList<QStringList> &)>(22, 7, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 23, 24 },
        }}),
        // Method 'updatePathInfo'
        QtMocHelpers::MethodData<void(const QString &, const QStringList &)>(25, 7, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::QStringList, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'count'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<NoteMapper, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject NoteMapper::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10NoteMapperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10NoteMapperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10NoteMapperE_t>.metaTypes,
    nullptr
} };

void NoteMapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NoteMapper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QVariantList _r = _t->getCleanedHeaderAndLevel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->saveMap(); break;
        case 2: { QList<QVariantMap> _r = _t->getNoteHeaders((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<QVariantMap>*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->addInitialGlobalPaths((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->addGlobalPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->updateGlobalPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: _t->removeGlobalPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->addLinkedNotesInfos((*reinterpret_cast<std::add_pointer_t<QList<QStringList>>>(_a[1]))); break;
        case 8: _t->updatePathInfo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QStringList> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->rowCount(); break;
        default: break;
        }
    }
}

const QMetaObject *NoteMapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NoteMapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10NoteMapperE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int NoteMapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
