/*
    SPDX-FileCopyrightText: 2026 Igor Mironchik <igor.mironchik@gmail.com>
    SPDX-License-Identifier: MIT
*/

// md4qt include.
#include "inline_context.h"

namespace MD
{

InlineContext::InlineContext()
{
}

} /* namespace MD */
